import urllib2
import json
import cache

transaction_url_format = 'https://blockchain.info/tx-index/%s?format=json'
address_url_format = 'http://blockchain.info/address/%s?format=json'

# Returns response text given an url
# Returns None if there was a problem
def request(request_url, parse=True):
    response = urllib2.urlopen(request_url).read()
    return parse_json(response) if parse else response

def parse_json(jsonstring):
    try:
        return json.loads(jsonstring)
    except ValueError:
        return None

def fetch_transaction(transaction_hash, parse=True):
    request_url = transaction_url_format % (transaction_hash)
    return request(request_url, parse)

def fetch_address(address_hash, parse=True):
    request_url = address_url_format % (address_hash)
    return request(request_url, parse)

AddressLookup = cache.Cache(lookup=fetch_address)
def address(address_hash):
    return AddressLookup.get_hash(address_hash)

TransactionLookup = cache.Cache(lookup=fetch_transaction)
def transaction(transaction_hash):
    return TransactionLookup.get_hash(transaction_hash)

def find_outputs_for_transaction(transaction_hash):
    return [output['addr'] for output in transaction(transaction_hash)]

def find_output_addresses_for_address(address_hash):
    jsonobject = address(address_hash)
    output_txs = jsonobject['txs']

    # Gives transactions, now get addresses from that
    output_transactions = [tx['out'] for tx in output_txs]

    all_output_addresses = []
    for outputs in output_transactions:
        output_addresses = [output['addr'] for output in outputs]
        all_output_addresses += output_addresses

    return all_output_addresses

def find_input_addresses_for_address(address_hash):
    jsonobject = address(address_hash)
    output_txs = jsonobject['txs']

    input_transactions = [tx['inputs'] for tx in output_txs]

    all_input_addresses = [] # All the sender addresses
    for inputs in input_transactions:
        input_addresses = [input['prev_out']['addr'] for input in inputs]
        all_input_addresses += input_addresses

    return all_input_addresses
