import sqlite3
import json

class Cache:
    def __init__(self, path='data/data.db', lookup=None):
        self.conn = sqlite3.connect(path)
        self.c = self.conn.cursor()
        self.lookup = lookup
        self.setup()

    def setup(self):
        self.c.execute('CREATE TABLE if not exists hashes (hash text, json text)')
        self.conn.commit()

    def get_hash(self, hash):
        self.c.execute('SELECT json FROM hashes WHERE hash=?', (hash,))
        result = self.c.fetchone()
        if result:
            return json.loads(result[0])
        elif self.lookup is not None:
            result = self.lookup(hash)
            if result:
                self.put_hash(hash, json.dumps(result))
                return result
        return None

    def put_hash(self, hash, json):
        self.c.execute('INSERT INTO hashes VALUES (?,?)', (hash, json))
        self.conn.commit()
