from collections import namedtuple
import networkx as nx
import graph
from utils import decimalize, pretty_print
import utils

AddrVal = namedtuple('AddrVal', 'addr,val')

# Take a known slush fund address in Bitcoinfog and analyze upward
def analyze_address_up(address):
    import api
    jsonobject = api.address(address)

    # The transaction with the input that is not address, is the deposit in here
    # Trace that go to upstream. If there are multiple, may have encountered
    # head of a slush fund - stop here

    deposits = []
    txs = jsonobject['txs']
    for tx in txs:
        # To see where money came from, look at transaction where address is not an input
        for input in tx['inputs']:
            if address not in input['prev_out']['addr']:
                deposits.append(tx)

    if len(deposits) > 1:
        print 'Came from multiple sources, probably mixing head address at ', address
        return None
    elif len(deposits) == 0:
        print 'No deposits in here' # Should only occur for mined addresses
        return None
    else:
        if len(deposits[0]['inputs']) > 1:
            print 'Complex, non bitcoinfog look like deposit at ', address
            return address
        print 'Deposit at ', address, ' amount: ', decimalize(deposits[0]['inputs'][0]['prev_out']['value'])
        deposit_addr = deposits[0]['inputs'][0]['prev_out']['addr'] # Should only be one input in bitcoinfog
        return deposit_addr

# Can find a slush fund's head based on an existing slush address, by working
# way up using analyze_address_up. After this, run chain_analyze_bitcoinfog
# from head to create larger graph
def find_slush_head(address):
    # #yolo
    if address in ['1Lz2Bg9KBUJq1f3osJRWYcogUkdSwBtwsk', '1AsHkGPVaJTXgqZmfW9mrwaeraFUBVj3WY']:
        return '1DH7mMoX5rTognAAbTnLrvhw4WXDxsYBfL'
    prev_address = address
    while address != None:
        prev_address = address
        address = analyze_address_up(address)
        if address == None: # not needed
            break

    print prev_address
    return prev_address

# Take an address, analyze where the withdrawals are (small amount) and the
# slush fund - large amount is (the main pot used by bitcoinfog for pay out)
# Useful for bitcoinfog transactions - give it the "slush fund" address to start
# out with. Can do this by looking up your withdrawal
def analyze_address(address):
    import api
    jsonobject = api.address(address)

    txs = jsonobject['txs']

    withdrawals = [] # Switch to list comprehenshion in future
    for tx in txs:
        # To see where money goes next, look at transaction where address is an input/previous out
        for input in tx['inputs']:
            if address == input['prev_out']['addr']:
                # Found one of the withdrawal transactions. Could be where money
                # goes to next. See which withdrawal has larger transaction to
                # trace to 'slush fund'
                withdrawals.append(tx)

        # If we find multiple withdrawals at one address, note this, since
        # This doesn't seem to be standard for bitcoinfog
    tx = None
    if len(withdrawals) == 0: # No withdrawals, we're ending here
        print 'Ended at address ', address, ' - no withdrawals'
        return []
    elif len(withdrawals) == 1: # Only one withdrawal, continue as normal
        tx = withdrawals[0]
    else:
        withdrawals.sort(key = lambda x: largest_output_of_tx(x))
        tx = withdrawals[0]

    # Larger transaction is the remainder of the slush fund
    # print 'Input addr: ', input['prev_out']['addr'], ': ', decimalize(input['prev_out']['value'])

    outputs = tx['out']
    outputs.sort(key = lambda x: x['value'])
    # Assume only two for now

    slush = outputs[-1]
    withdrawal = outputs[0]

    if slush == withdrawal:
        print 'CYCLE!!!'

    # print 'Output/slush fund: ', outputs[1]['addr'], ': ', decimalize(outputs[1]['value'])
    print 'Withdrawal: ', outputs[0]['addr'], ': ', decimalize(outputs[0]['value']), '. Slush: ', slush['addr'], ': ', decimalize(slush['value'])

    # Smaller transaction is the withdrawal from bitcoin fog - keep track of these
    return (AddrVal(addr=slush['addr'], val=decimalize(slush['value'])),
        AddrVal(addr=withdrawal['addr'], val=decimalize(withdrawal['value'])))
    # Looking at the transaction where address is an output, tells you where money came from

def largest_output_of_tx(tx):
    outputs = tx['out']
    outputs.sort(key = lambda x: x['value'])
    return outputs[-1]

# Returns a networkx graph
def chain_analyze_bitcoinfog(start_address, steps):
    address = start_address

    G = nx.DiGraph()
    G.add_node(start_address, type='start')

    for i in range(steps):
        out = analyze_address(address)
        if len(out) == 0:
            break

        slush = out[0]
        withdrawal = out[1]

        G.add_node(slush.addr, type='slush', order=i)
        G.add_node(withdrawal.addr, type='withdrawal', order=i)

        G.add_edge(address, slush.addr, type='slush', amount=slush.val, order=i)
        G.add_edge(address, withdrawal.addr, type='withdrawal', amount=withdrawal.val, order=i)

        address = slush.addr

    return G

def extend_withdrawal_graph(G):
    import api
    H = nx.DiGraph()

    H.add_node('start', type='start')

    total = set()
    withdrawals = [v for (v,d) in G.nodes(data=True) if d['type'] == 'withdrawal']
    for withdrawal in withdrawals:
        H.add_edge('start', withdrawal, type='start')
        output_addresses = api.find_output_addresses_for_address(withdrawal)
        for output_address in output_addresses:
            total.add(output_address)
            H.add_node(output_address, type='normal')
            H.add_node(withdrawal, type='withdrawal')
            H.add_edge(withdrawal, output_address, type='normal')
    print len(total)

    return H

import re
from collections import Counter

def get_sizes_from_graph(G):
    withdrawal_sizes = [d['amount'] for (u,v,d) in G.edges(data=True) if d['type'] == 'withdrawal']
    return withdrawal_sizes

def get_sizes_from_address(address, input=True, output=True):
    jsonobject = api.address(address)

    amounts = []
    txs = jsonobject['txs']
    for tx in txs:
        if input:
            amounts.extend(decimalize(input['prev_out']['value']) for input in tx['inputs'])
        if output:
            amounts.extend(decimalize(out['value']) for out in tx['out'])

    return amounts

# Given a list of transaction sizes, find its first digits
def get_digit_distribution(transaction_sizes):
    first_digit = []
    # Get the first nonzero digit out of a float
    for t in transaction_sizes:
        # digits = str(n)
        nonZeroDigit = re.search('[123456789]', str(t))
        if nonZeroDigit:
            first_digit.append(nonZeroDigit.group(0))
        else:
            print 'Something is wrong here: All transactions should have a nonzero size'
            return []
    c = Counter(first_digit)

    tot = len(transaction_sizes)

    digits = {}
    for digit in xrange(1,10):
        digits[digit] = c[str(digit)] / float(tot)

    return digits

# Given a list of transaction sizes, find its second digit distribution
def get_second_digit_distribution(transaction_sizes):
    second_digit = []
    # Get the first nonzero digit out of a float
    for t in transaction_sizes:
        s = str(t)
        nonZeroDigit = re.search('[123456789]', s)
        if nonZeroDigit:
            idx = nonZeroDigit.start(0)
            if idx == len(s) - 1:
                second_digit.append('0')
            else:
                if s[idx + 1] == '.':
                    second_digit.append(s[idx + 2])
                else:
                    second_digit.append(s[idx + 1])
        else:
            print 'Something is wrong here: All transactions should have a nonzero size'
            return []
    c = Counter(second_digit)

    print c

    tot = len(transaction_sizes)

    digits = {}
    for digit in xrange(0,10):
        digits[digit] = c[str(digit)] / float(tot)

    return digits

if __name__ == '__main__':
    s = '1Fn2VowDBZ6RRBDRxUC4RiToQhAiCnag4X'
    G = chain_analyze_bitcoinfog(s, 1000)
