import matplotlib.pyplot as plot
import networkx as nx
from networkx.readwrite import json_graph
import json

def graph_slush_withdrawal(G, start_address):
    edges = [(u,v) for (u,v,d) in G.edges(data=True)]# if d['type'] == 'slush']
    slushes = [v for (v,d) in G.nodes(data=True) if d['type'] == 'slush']
    withdrawals = [v for (v,d) in G.nodes(data=True) if d['type'] == 'withdrawal']

    pos = nx.spring_layout(G)
    nx.draw_networkx_nodes(G, pos, nodelist=[start_address], node_size=100, node_color='r')
    nx.draw_networkx_nodes(G, pos, nodelist=slushes, node_size=100, node_color='b')
    nx.draw_networkx_nodes(G, pos, nodelist=withdrawals, node_size=50, node_color='g')
    nx.draw_networkx_edges(G, pos, edgelist=edges, width=1, arrows=False)

    # nx.draw_networkx_labels(G, pos, font_size=8, font_family='sans-serif')

    plot.axis('off')
    plot.savefig('tmp/graph.png') # save as png
    plot.show()

def graph_withdrawal(G):
    pos = nx.fruchterman_reingold_layout(G, k=50.0)

    normal = [v for (v,d) in G.nodes(data=True) if d['type'] == 'normal']
    nx.draw_networkx_nodes(G, pos, nodelist=normal, node_size=100, node_color='b')

    withdrawals = [v for (v,d) in G.nodes(data=True) if d['type'] == 'withdrawal']
    nx.draw_networkx_nodes(G, pos, nodelist=withdrawals, node_size=50, node_color='g')

    edges = [(u,v) for (u,v,d) in G.edges(data=True) if d['type'] == 'normal']
    nx.draw_networkx_edges(G, pos, edgelist=edges, width=1)

    plot.axis('off')
    plot.savefig('tmp/graph.png') # save as png
    plot.show()

def export_json(G, filepath=None):
    d = json_graph.node_link_data(G)
    if filepath:
        json.dump(d, open(filepath, 'w'), indent=2)
    return d
